/*
 * Sweep, a sound wave editor.
 *
 * Copyright (C) 2000 Conrad Parker
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <string.h>
#include <glib.h>
#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include <sweep/sweep_i18n.h>
#include <sweep/sweep_types.h>
#include <sweep/sweep_sample.h>

static void
question_dialog_destroy_cb (GtkWidget * widget, gpointer data)
{
  GtkWidget * dialog;
  sw_sample * sample;

  dialog = gtk_widget_get_toplevel (widget);

  sample = gtk_object_get_user_data (GTK_OBJECT(dialog));
  if (sample && sample_bank_contains (sample)) {
    sample_set_edit_mode (sample, SWEEP_EDIT_MODE_READY);
  }
}

static void
question_dialog_answer_cb (GtkWidget * widget, gpointer data)
{
  GtkWidget * dialog;
  GtkSignalFunc func;

  func = gtk_object_get_user_data (GTK_OBJECT(widget));
  dialog = gtk_widget_get_toplevel (widget);

  if (func != NULL)
    func (widget, data);

  /* "destroy" will call destroy above */
  gtk_widget_destroy (dialog);
}

static void
query_dialog_new (sw_sample * sample, char * title, char * question,
		  gboolean show_cancel, char * ok_answer, char * no_answer,
		  GtkSignalFunc ok_callback, gpointer ok_callback_data,
		  GtkSignalFunc no_callback, gpointer no_callback_data)
{
  GtkWidget * window;
  GtkWidget * button, * ok_button;
  GtkWidget * vbox;
  GtkWidget * label;
  GtkAccelGroup * accel_group;

  window = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW(window), title);
  gtk_window_position(GTK_WINDOW(window), GTK_WIN_POS_MOUSE);
  gtk_container_border_width (GTK_CONTAINER(window), 8);

  gtk_object_set_user_data (GTK_OBJECT(window), sample);

  accel_group = gtk_accel_group_new ();
  gtk_window_add_accel_group (GTK_WINDOW(window), accel_group);

  gtk_signal_connect (GTK_OBJECT(window), "destroy",
		      GTK_SIGNAL_FUNC(question_dialog_destroy_cb), window);

  gtk_accel_group_add (accel_group, GDK_w, GDK_CONTROL_MASK, GDK_NONE,
		       GTK_OBJECT(window), "destroy");

  vbox = GTK_DIALOG(window)->vbox;

  /* Question */

  label = gtk_label_new (question);
  gtk_box_pack_start (GTK_BOX(vbox), label, FALSE, FALSE, 0);
  gtk_widget_show (label);

  /* OK */

  if (ok_answer == NULL) ok_answer = _("OK");
  ok_button = gtk_button_new_with_label (ok_answer);
  GTK_WIDGET_SET_FLAGS (GTK_WIDGET (ok_button), GTK_CAN_DEFAULT);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window)->action_area),
		      ok_button, TRUE, TRUE, 0);
  gtk_object_set_user_data (GTK_OBJECT(ok_button), ok_callback);
  gtk_widget_show (ok_button);
  gtk_signal_connect (GTK_OBJECT(ok_button), "clicked",
		      GTK_SIGNAL_FUNC (question_dialog_answer_cb),
		      ok_callback_data);

  /* Cancel */
  
  if (show_cancel) {
    if (no_answer == NULL) no_answer = _("Cancel");
    button = gtk_button_new_with_label (no_answer);
    GTK_WIDGET_SET_FLAGS (GTK_WIDGET (button), GTK_CAN_DEFAULT);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window)->action_area),
			button, FALSE, FALSE, 0);
    gtk_object_set_user_data (GTK_OBJECT(button), no_callback);
    gtk_widget_show (button);
    gtk_signal_connect (GTK_OBJECT(button), "clicked",
			GTK_SIGNAL_FUNC (question_dialog_answer_cb),
			no_callback_data);
  }

  gtk_widget_grab_default (ok_button);

  gtk_widget_show (window);
}

void
question_dialog_new (sw_sample * sample, char * title, char * question,
		     char * yes_answer, char * no_answer,
		     GtkSignalFunc yes_callback, gpointer yes_callback_data,
		     GtkSignalFunc no_callback, gpointer no_callback_data,
		     sw_edit_mode edit_mode)
{
  if (edit_mode != SWEEP_EDIT_MODE_READY)
    sample_set_edit_mode (sample, edit_mode);

  query_dialog_new (sample, title, question, TRUE, yes_answer, no_answer,
		    yes_callback, yes_callback_data,
		    no_callback, no_callback_data);
}

void
info_dialog_new (char * title, char * message)
{
  query_dialog_new (NULL, title, message, FALSE, _("OK"), NULL, NULL, NULL,
		    NULL, NULL);
}
